package ru.inils.utils.textTable {
	/**
	 * @author	iNils
	 * @version	1.0
	 *
	 * Класс строит текстовую таблицу данных.
	 * 
	 * @example <code>
	 * /// Создае новую таблицу
	 * var tt:TextTable = new TextTable (['ID', 'Number', 'Name', 'Year']);
	 * /// Добавляем выравнивание по правому края для второй колонки.
	 * tt.alignRight = [ , true];
	 * /// Добавляем 4 строку и разделитель.
	 * tt.addRow ([1, 100, 'Tom', 1966]);
	 * tt.addRow ([2, 2, 'Anna', 1985]);
	 * tt.addSeparator ();
	 * tt.addRow ([3, 55, 'Serg', 2001]);
	 * tt.addRow ([4, 4701, 'Nika', 1991]);
	 * /// Результат
	 * trace (tt.table);
	 * +----+--------+------+------+
	 * | ID | Number | Name | Year |
	 * +----+--------+------+------+
	 * | 1  |    100 | Tom  | 1966 |
	 * | 2  |      2 | Anna | 1985 |
	 * +----+--------+------+------+
	 * | 3  |     55 | Serg | 2001 |
	 * | 4  |   4701 | Nika | 1991 |
	 * +----+--------+------+------+ 
	 * 
	 * /// Отчищаем таблицу от данных.
	 * tt.clear ();
	 * /// Заполняем новыми данными
	 * tt.addRow ([5, 46, 'Paul', 1981]);
	 * tt.addRow ([6, 4, 'Victoria', 1975]);
	 * tt.addSeparator ();
	 * tt.addRow ([7, , 'Arnold', 1989]);
	 * tt.addRow ([8, 956, 'Lena', 1998, 55]);
	 * /// Результат
	 * trace (tt.table);
	 * +----+--------+----------+------+
	 * | ID | Number | Name     | Year |
	 * +----+--------+----------+------+
	 * | 5  |     46 | Paul     | 1981 |
	 * | 6  |      4 | Victoria | 1975 |
	 * +----+--------+----------+------+
	 * | 7  |   null | Arnold   | 1989 |
	 * | 8  |    956 | Lena     | 1998 >
	 * +----+--------+----------+------+ 
	 * </code>
	 * 
	 * Изменения:
	 * 	1.0 (24.01.2011 22:32)
	 *	[+]	...
	 */
	public class TextTable {
		/*  - VAR PRIVATE        *///{ /
		private var _alignRight:Array = [];
		private var _head:Array = [];
		private var _color/*int*/:Array = [];
		private var _row:Array = [];
		private var _size:Array = [];
		//}
		/*  * CONSTRUCTOR *      *///{ /
		/**
		 * Создает новую текстовую таблицу.
		 * @param	head - Массив названий столбцов таблицы.
		 * @param	color - Цвет заголовка таблицы (для вывода в htmlText поле).
		 */
		public function TextTable (head:Array, color:int = -1) {
			_head = head;
			addRow (_head, color);
		}
		//}
		/*  + METHOD PUBLIC      *///{ /
		/**
		 * Добавляет строку данных в таблицу. Если число элементов в строке больше числа столбцов, то строка заканчивается символом '>', а не '|'.
		 * @param	row - Строка таблицы.
		 * @param	color - Цвет строки таблицы (для вывода в htmlText поле).
		 */
		public function addRow (row:Array, color:int = -1):void {
			_row.push (row);
			_color.push (Math.max (-1, Math.min (color, 0xFFFFFF)));
			checkSize ();
		}
		/**
		 * Добавляет разделитель между строками.
		 * @param	color - Цвет разделителя (для вывода в htmlText поле).
		 */
		public function addSeparator (color:int = -1):void {
			_row.push (null);
			_color.push (Math.max (-1, Math.min (color, 0xFFFFFF)));
			checkSize ();
		}
		/**
		 * Отчищает таблицу от данных, оставляя заголовок таблицы и выравнивание данных.
		 */
		public function clear ():void {
			_row.length = 1;
			_color.length = 1;
			_size.length = 0;
			//addRow (_head, );
			checkSize ();
		}
		//}
		/*  - METHOD PRIVATE     *///{ /
		/**
		 * Строит текстовую таблицу.
		 * @return Текстовую таблицу в виде массива строк.
		 */
		private function buildTable (htmlMode:Boolean = true):Array {
			var array:Array = [];
			
			/// шаблоны строк
			var line:String = '+';
			for (var i:int = 0; i < _head.length; i++) {
				line += fillChars ('', _size[i], true, '-') + '+';
			}
			
			/// заполняем данными
			var row:String = ''
			var j:int;
			for (i = 0; i < _row.length; i++) {
				var isColor:Boolean = _color[i] != -1 && htmlMode;
				var color:String = '<font color="#' + fillChars (_color[i].toString (16), 6, false, '0') + '">';
				/// заголовок
				if (i == 0) {
					if (isColor) {
						row += color;
					}
					row += line + '\r' + '|'
					for (j = 0; j < _head.length; j++) {
						row += ' ' + fillChars (_row[i][j], _size[j] - 2) + ' |';
					}
					row += '\r' + line;
					if (isColor) {
						row += '</font>'
					}
					row += '\r';
					array.push (row);
				} else {
					row = '';
					/// строки
					if (_row[i]) {
						if (isColor) {
							row += color;
						}
						row += '|'
						for (j = 0; j < _head.length; j++) {
							if (j != 0) {
								row += '|';
							}
							row += ' ' + fillChars (_row[i][j], _size[j] - 2, !_alignRight[j]) + ' ';
						}
						if (_head.length < _row[i].length) {
							row += '>';
						} else {
							row += '|';
						}
						if (isColor) {
							row += '</font>'
						}
						array.push (row + '\r');
					} else {
						if (isColor) {
							row += color;
						}
						row += line;
						if (isColor) {
							row += '</font>'
						}
						array.push (row + '\r');
					}
				}
			}
			array.push (line + '\r');
			return array;
		}
		/**
		 * Вычисляет наибольшую ширину столбца.
		 */
		private function checkSize ():void {
			var array:Array = _row[_row.length - 1];
			if (!array) {
				return;
			}
			for (var i:int = 0; i < _head.length; i++) {
				var len:int = String (array[i] || '').length;
				if (len > 0) {
					len += 2;
				}
				_size[i] = Math.max (_size[i] || 0, len);
			}
		}
		/**
		 * Заполняет строку до заданной длины символом слева или справа.
		 * @param	data - Искходная строка.
		 * @param	len - Длина до которой нужно довести строку.
		 * @param	fillRight - Место заполнения символами, справа от строки (true) или слева (false).
		 * @param	char - Символ заполнения.
		 * @return
		 */
		private function fillChars (data:Object, len:int, fillRight:Boolean = true, char:String = ' '):String {
			var str:String = String (data);
			len -= str.length;
			var out:String = '';
			if (fillRight) {
				out += str;
			}
			for (var i:int = 0; i < len; i++) {
				out += char;
			}
			if (!fillRight) {
				out += str;
			}
			return out;
		}
		//}
		/*    GETTER / SETTER    *///{ /
		/// Массив выравнивания данных по правой (true) или левой стороне (false).
		public function get alignRight ():Array {
			return _alignRight;
		}
		public function set alignRight (value:Array):void {
			if (value == _alignRight) {
				return;
			}
			_alignRight = value;
		}
		/// Текстовая таблица в виде строки.
		public function get table ():String {
			return buildTable ().join ('');
		}
		/// Текстовая таблица в виде массива строк без тегов html.
		public function get tableArray ():Array {
			return buildTable (false);
		}
		/// Текстовая таблица в виде строки без тегов html.
		public function get tableText ():String {
			return buildTable (false).join ('');
		}
		//}
	}
}
